require "active_support/hash_with_indifferent_access"

class Dilute::Attribute
  attr_reader :options
  def initialize(options = {})
    @options = HashWithIndifferentAccess.new.merge(options)
  end

  %w(index_name store index term_vector boost null_value omit_norms
     omit_term_freq_and_positions index_options analyzer index_analyzer
     search_analyzer include_in_all ignore_above position_offset_gap
     precision_step ignore_malformed format).each do |option|
    define_method option do |new_value|
      raise "invalid option #{option} for #{type}" unless acceptable_mapping_options[type_mapping].include?(option)
      chain_add(option => new_value)
    end
  end

  def not_indexed
    index("not_analyzed")
  end

  def to_config
    options
  end

  private

  def chain_add(hsh)
    # self.class.new(options.merge(hsh))
    options.merge!(hsh)
    self
  end

  def type(new_val = nil)
    new_val ? options[:type] = new_val.to_sym : options[:type].to_sym
  end

  def type_mapping
    return :number if [:float, :double, :integer, :long, :short, :byte].include?(type)
    type
  end


  def acceptable_mapping_options
    {
      string: %w(index_name store index term_vector boost null_value omit_norms omit_term_freq_and_positions index_options analyzer index_analyzer search_analyzer include_in_all ignore_above position_offset_gap),
      number: %w(type index_name store index precision_step boost null_value include_in_all ignore_malformed),
      date: %w(index_name format store index precision_step boost null_value include_in_all ignore_malformed),
      boolean: %w(index_name store index boost null_value include_in_all),
      binary: %w(index_name)
    }
  end
end
