require "active_support/concern"
require "active_model/naming"
require "active_support/core_ext/string"
require "active_model/callbacks"
require "active_model/validations"
require "active_model/translation"


module Dilute::Modelize
  extend ActiveSupport::Concern

  included do
    attr_reader :attributes
    extend  ActiveModel::Naming
    extend  ActiveModel::Callbacks
    include ActiveModel::Validations
    define_model_callbacks :save
  end

  module ClassMethods
    def refresh
      type.server.refresh
    end

    def name
      super || "DefaultDilutedModel"
    end

    def find(id)
      new(type.get(id, {}, true))
    end

    def all(query = {}, page_size = 10)

      # search_results = type.search(size: page_size, query: {match_all: {}})
      # search_results.raw["hits"]["hits"].collect {|r| new(r) }
      Dilute::Query.new(self)
    end

    def define_type(options = {}, &blk)
      @type ||= Dilute::Type.new(type_defaults.merge(options), &blk)
      add_attribute_accessors
    end

    def application_name
      if Kernel.const_defined? :Rails
        Rails.application.class.parent_name
      else
        "DefaultDiluteIndexName"
      end
    end

    def type_defaults
      HashWithIndifferentAccess.new({
        index_name: application_name,
        type_name: model_name.plural
      })
    end

    def add_attribute_accessors
      type.properties.keys.each do |k|
        define_method k do
          attributes["_source"] && attributes["_source"][k]
        end
        define_method "#{k}=" do |new_var|
          attributes["_source"] && attributes["_source"][k] = new_var
        end
      end
    end

    def type
      @type or raise "No type defined!"
    end
  end

  def id
    attributes["_id"]
  end

  def attributes_without_elasticsearch_vars
    # elasticsearch_vars = %w(ok _index _type _id _version)
    attributes["_source"]
  end

  def save
    run_callbacks :save do
      if valid?
        if id
          type.put(id, attributes_without_elasticsearch_vars)
        else
          results = type.post(attributes_without_elasticsearch_vars)
          if results["ok"] == true
            attributes.merge!(results)
          else
            puts "Something went wrong!"
            puts "attributes: #{attributes}"
          end
        end
      end
    end
  end

  def destroy
    raise "id can't be nil" unless id
    type.delete(id)
  end

  def initialize(attributes = {})
    @attributes = if attributes.has_key?("_source")
      attributes
    else
      {"_source" => HashWithIndifferentAccess.new.merge(attributes)}
    end
  end

  def type
    self.class.type.type
  end

end
