class Dilute::Query
  attr_reader :for_class, :options, :chained, :results, :from_elastic_search
  include Enumerable

  def initialize(for_class, options = {}, chained = true)
    @for_class, @options, @chained = for_class, options, chained
  end

  def match(q)
    merge_or_chain(:match, q)
  end

  def to_query
    query = options.reject {|k,v| v.nil? || v.empty? }
    query[:match_all] ||= {} unless query[:match]
    query
  end

  def execute!
    @results ||= begin
      search_results = for_class.type.search(size: 25, query: to_query)
      @from_elastic_search = search_results
      search_results.raw["hits"]["hits"].collect {|r| for_class.new(r) }
    end
  end

  def to_a
    execute!
    results
  end

  def each(*args, &block)
    return enum_for(__callee__) unless block_given?
    to_a.each(*args, &block)
  end

  private

  def merge_or_chain(key, to_merge)
    new_options = chained ? options.dup : options
    new_options[key] ||= {}
    new_options[key].merge!(to_merge)
    chained ? self.class.new(for_class, new_options) : new_options[key]
  end
end