require "active_support/hash_with_indifferent_access"
require "stretcher"

class Dilute::Type
  attr_reader :options, :attributes
  def initialize(options = {}, &block)
    @options = HashWithIndifferentAccess.new.merge(options)
    @attributes = HashWithIndifferentAccess.new
    instance_eval(&block) if block_given?
  end

  def server_url
    options[:server_url] || "http://localhost:9200"
  end

  def server
    @server ||= Stretcher::Server.new(server_url, log_level: :info)
  end

  def index_name
    # Ugly
    index_name = "#{options[:index_name]}".gsub(/([A-Z])/) { "_#{$1.downcase}"}.sub(/\A_/,'')
  end

  def type_name
    options[:type_name]
  end

  def index
    @index ||= begin
      the_index = server.index(index_name)
      the_index.exists? ? the_index : (the_index.create && the_index)
    end
  end

  def type
    @type ||= begin
      the_type = index.type(type_name)
      the_type.exists? ? the_type : (the_type.put_mapping(generate_mapping) && the_type)
    end
  end

  def delete
    result = type.delete_mapping
    @type = nil
    result
  end

  def generate_mapping
    { :"#{type_name}" => { properties: properties } }
  end

  def properties
    attributes.inject({}) do |sum, pair|
      k, v = pair
      sum[k] = v.to_config ; sum
    end
  end

  def mapping
    type.get_mapping
  end

  def attribute(key, attribute_type = :string)
    attributes[key] = Dilute::Attribute.new(type: attribute_type)
  end

  def search(*args)
    type.search *args
  end

  def get(*args)
    type.get *args
  end

end